Package: JLPM
Type: Package
Title: Joint Latent Process Models
Version: 1.0.4
Date: 2026-04-30
Authors@R: c(person("Viviane", "Philipps", role=c("aut", "cre"), email="Viviane.Philipps@u-bordeaux.fr"),
             person("Tiphaine", "Saulnier", role="aut"),
	     person("Cecile", "Proust-Lima", role="aut"))
Description: Estimation of extended joint models with shared random effects. Longitudinal data are handled in latent process models for continuous (Gaussian or curvilinear) and ordinal outcomes while proportional hazard models are used for the survival part. We propose a frequentist approach using maximum likelihood estimation. See Saulnier et al, 2022 <doi:10.1016/j.ymeth.2022.03.003>.
License: GPL (>= 2.0)
Depends: R (>= 3.5.0), lcmm
Imports: survival (>= 2.37-2), spacefillr, stringr, marqLevAlg (>=
        2.0.6)
URL: https://VivianePhilipps.github.io/JLPM/
BugReports: https://github.com/VivianePhilipps/JLPM/issues
LazyLoad: yes
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-04 08:18:52 UTC; vp3
Author: Viviane Philipps [aut, cre],
  Tiphaine Saulnier [aut],
  Cecile Proust-Lima [aut]
Maintainer: Viviane Philipps <Viviane.Philipps@u-bordeaux.fr>
Repository: CRAN
Date/Publication: 2026-05-04 13:00:13 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-04 17:02:41 UTC; unix
Archs: JLPM.so.dSYM
