# Generated by roxygen2: do not edit by hand

S3method(coef,jointLPM)
S3method(print,jointLPM)
S3method(summary,jointLPM)
S3method(vcov,jointLPM)
export(IRT4Sselecting)
export(IRT4Sstaging)
export(REconddensity)
export(WaldMult)
export(convert)
export(createFargs)
export(createX0)
export(diffSojournTime)
export(jointLPM)
export(loglik)
export(predictRE)
export(removeNA)
export(sojournTime)
importFrom(grDevices,col2rgb)
importFrom(grDevices,n2mfrow)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(lcmm,Jointlcmm)
importFrom(lcmm,multlcmm)
importFrom(lcmm,predictYcond)
importFrom(marqLevAlg,mla)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,clusterSetRNGStream)
importFrom(parallel,makeCluster)
importFrom(parallel,parApply)
importFrom(parallel,stopCluster)
importFrom(spacefillr,generate_sobol_owen_set)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,delete.response)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,get_all_vars)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.action)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(stringr,str_detect)
importFrom(survival,Surv)
importFrom(survival,untangle.specials)
importFrom(utils,as.roman)
useDynLib(JLPM, .registration=TRUE, .fixes="C_")
