## Installation


To install the CRAN version, use :

``` r
install.packages("JLPM")
```

To get the most recent update, install it from github :

``` r
remotes::install_github("VivianePhilipps/JLPM")
```


## Documentation


Some vignettes are available on the JLPM website <https://VivianePhilipps.github.io/JLPM/>.
