% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ForInternalUse.R
\name{ForInternalUse}
\alias{ForInternalUse}
\alias{.expect}
\alias{.Pim}
\alias{.Pim_prime}
\alias{.integrate_info}
\title{For internal use only ...}
\description{
For internal use only ...
}
